#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec2 textureCoordinate;
uniform samplerExternalOES s_texture;
uniform highp vec2 singleStepOffset;

highp vec2 blurCoordinates[20];

uniform highp float radius;
uniform highp float eyeRadius;
uniform highp float eyeScaleRatio;
uniform highp float aspectRatio;
uniform highp vec2 leftEyeCenterPosition;
uniform highp vec2 rightEyeCenterPosition;
const highp int MAX_CONTOUR_POINT_COUNT = 2;
uniform float leftContourPoints[MAX_CONTOUR_POINT_COUNT * 2];
uniform float rightContourPoints[MAX_CONTOUR_POINT_COUNT * 2];
uniform float deltaArray[MAX_CONTOUR_POINT_COUNT];
uniform int arraySize;
uniform int faceCount;
uniform int shouldBeauty;

/** 磨皮等级 0 ~ 1 之间*/
const float opacity = 0.7;

highp vec2 warpPositionToUseEye(vec2 centerPostion, vec2 currentPosition, float radius, float scaleRatio, float aspectRatio)
{
     vec2 positionToUse = currentPosition;
     vec2 currentPositionToUse = vec2(currentPosition.x * aspectRatio * 2.0 + 0.5 - aspectRatio, currentPosition.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
     vec2 centerPostionToUse = vec2(centerPostion.x * aspectRatio * 2.0 + 0.5 - aspectRatio, centerPostion.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
     float r = distance(currentPositionToUse, centerPostionToUse);
     if(r < radius)
     {
          float alpha = 1.0 - scaleRatio * pow(r / radius - 1.0, 2.0);
          positionToUse = centerPostion + alpha * (currentPosition - centerPostion);
//     } else if (r - radius < 0.01) { // test
//          positionToUse = vec2(0.0, 0.0);
     }
     return positionToUse;
}
highp vec2 warpPositionToUse(vec2 currentPoint, vec2 contourPointA,  vec2 contourPointB, float radius, float delta, float aspectRatio)
{
     vec2 positionToUse = currentPoint;
     vec2 currentPointToUse = vec2(currentPoint.x, currentPoint.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
     vec2 contourPointAToUse = vec2(contourPointA.x, contourPointA.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
     float r = distance(currentPointToUse, contourPointAToUse);
     if(r < radius)
     {
          vec2 dir = normalize(contourPointB - contourPointA);
          float dist = radius * radius - r * r;
          float alpha = dist / (dist + (r-delta) * (r-delta));
          alpha = alpha * alpha;
          positionToUse = positionToUse - alpha * delta * dir;
//     } else if (r - radius < 0.01) { // test
//          positionToUse = vec2(0.0, 0.0);
     }
     return positionToUse;
}

highp float hardLight(highp float color) {
     if (color <= 0.5)
         color = color * color * 2.0;
     else
         color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
     return color;
}


void main() {
    highp vec2 positionToUse = textureCoordinate;
    if (faceCount > 0) {
        for(int i = 0; i < arraySize; i++) {
            positionToUse = warpPositionToUse(positionToUse, vec2(leftContourPoints[i * 2], leftContourPoints[i * 2 + 1]), vec2(rightContourPoints[i * 2], rightContourPoints[i * 2 + 1]), radius, deltaArray[i], aspectRatio);
            positionToUse = warpPositionToUse(positionToUse, vec2(rightContourPoints[i * 2], rightContourPoints[i * 2 + 1]), vec2(leftContourPoints[i * 2], leftContourPoints[i * 2 + 1]), radius, deltaArray[i], aspectRatio);
        }
        positionToUse = warpPositionToUseEye(leftEyeCenterPosition, positionToUse, eyeRadius, eyeScaleRatio, aspectRatio);
        positionToUse = warpPositionToUseEye(rightEyeCenterPosition, positionToUse, eyeRadius, eyeScaleRatio, aspectRatio);
    }
    if (shouldBeauty <= 0) {
        gl_FragColor = texture2D(s_texture, positionToUse);
    } else {
        highp vec3 centralColor = texture2D(s_texture, positionToUse).rgb;

        float x_a = float(singleStepOffset.x);
        float y_a = float(singleStepOffset.y);

        float mul_x = 2.0 / x_a;
        float mul_y = 2.0 / y_a;
        blurCoordinates[0] = textureCoordinate + vec2(0.0 * mul_x, -10.0 * mul_y);
        blurCoordinates[2] = textureCoordinate + vec2(8.0 * mul_x, -5.0 * mul_y);
        blurCoordinates[4] = textureCoordinate + vec2(8.0 * mul_x, 5.0 * mul_y);
        blurCoordinates[6] = textureCoordinate + vec2(0.0 * mul_x, 10.0 * mul_y);
        blurCoordinates[8] = textureCoordinate + vec2(-8.0 * mul_x, 5.0 * mul_y);
        blurCoordinates[10] = textureCoordinate + vec2(-8.0 * mul_x, -5.0 * mul_y);

        mul_x = 1.8 / x_a;
        mul_y = 1.8 / y_a;
        blurCoordinates[1] = textureCoordinate + vec2(5.0 * mul_x, -8.0 * mul_y);
        blurCoordinates[3] = textureCoordinate + vec2(10.0 * mul_x, 0.0 * mul_y);
        blurCoordinates[5] = textureCoordinate + vec2(5.0 * mul_x, 8.0 * mul_y);
        blurCoordinates[7] = textureCoordinate + vec2(-5.0 * mul_x, 8.0 * mul_y);
        blurCoordinates[9] = textureCoordinate + vec2(-10.0 * mul_x, 0.0 * mul_y);
        blurCoordinates[11] = textureCoordinate + vec2(-5.0 * mul_x, -8.0 * mul_y);

        mul_x = 1.6 / x_a;
        mul_y = 1.6 / y_a;
        blurCoordinates[12] = textureCoordinate + vec2(0.0 * mul_x, -6.0 * mul_y);
        blurCoordinates[14] = textureCoordinate + vec2(-6.0 * mul_x, 0.0 * mul_y);
        blurCoordinates[16] = textureCoordinate + vec2(0.0 * mul_x, 6.0 * mul_y);
        blurCoordinates[18] = textureCoordinate + vec2(6.0 * mul_x, 0.0 * mul_y);

        mul_x = 1.4 / x_a;
        mul_y = 1.4 / y_a;
        blurCoordinates[13] = textureCoordinate + vec2(-4.0 * mul_x, -4.0 * mul_y);
        blurCoordinates[15] = textureCoordinate + vec2(-4.0 * mul_x, 4.0 * mul_y);
        blurCoordinates[17] = textureCoordinate + vec2(4.0 * mul_x, 4.0 * mul_y);
        blurCoordinates[19] = textureCoordinate + vec2(4.0 * mul_x, -4.0 * mul_y);

        float central;
        float gaussianWeightTotal;
        float sum;
        float sampler;
        float distanceFromCentralColor;
        float gaussianWeight;

        float distanceNormalizationFactor = 3.6;

        central = texture2D(s_texture, textureCoordinate).g;
        gaussianWeightTotal = 0.2;
        sum = central * 0.2;

        sampler = texture2D(s_texture, blurCoordinates[0]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[1]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[2]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[3]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[4]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[5]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[6]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[7]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[8]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[9]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[10]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[11]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.09 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[12]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[13]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[14]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[15]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[16]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[17]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[18]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sampler = texture2D(s_texture, blurCoordinates[19]).g;
        distanceFromCentralColor = min(abs(central - sampler) * distanceNormalizationFactor, 1.0);
        gaussianWeight = 0.1 * (1.0 - distanceFromCentralColor);
        gaussianWeightTotal += gaussianWeight;
        sum += sampler * gaussianWeight;

        sum = sum/gaussianWeightTotal;

        sampler = centralColor.g - sum + 0.5;

        /** 高反差保留*/
        for (int i = 0; i < 5; ++i) {
            sampler = hardLight(sampler);
        }

        float aa = 1.0 + pow(sum, 0.3) * 0.09;
        highp vec3 smoothColor = centralColor * aa - vec3(sampler) * (aa - 1.0);
        smoothColor = clamp(smoothColor, vec3(0.0), vec3(1.0));

        smoothColor = mix(centralColor, smoothColor, pow(centralColor.g, 0.33));
        smoothColor = mix(centralColor, smoothColor, pow(centralColor.g, 0.39));

        smoothColor = mix(centralColor, smoothColor, opacity);
        gl_FragColor = vec4(pow(smoothColor, vec3(0.96)), 1.0);

    }
}